﻿using System;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using UnityEngine;

namespace VaMJapan
{
    public class PoseMirroring : MVRScript
    {
        private SuperController sc = SuperController.singleton;
        private JSONStorableStringChooser MirroringKey;
        private JSONStorableStringChooser CenterPoint;
        private JSONStorableStringChooser Source;
        private JSONStorableStringChooser Axis;
        private JSONStorableStringChooser PosX;
        private JSONStorableStringChooser PosY;
        private JSONStorableStringChooser PosZ;
        private JSONStorableStringChooser RotateX;
        private JSONStorableStringChooser RotateY;
        private JSONStorableStringChooser RotateZ;
        private JSONStorableBool MirrorParam;

        List<string> ControlNames = new List<string>()  {
            "control",
            "head",
            "neck",
            "chest",
            "pelvis",
            "hip",
            "andomen",
            "andomen2",
            "headControl",
            "neckControl",
            "chestControl",
            "pelvisControl",
            "hipControl",
            "abdomenControl",
            "abdomen2Control",
            "rHandControl",
            "lHandControl",
            "rFootControl",
            "lFootControl",
            "eyeTargetControl",
            "rNippleControl",
            "lNippleControl",
            "rElbowControl",
            "lElbowControl",
            "rKneeControl",
            "lKneeControl",
            "rToeControl",
            "lToeControl",
            "rThighControl",
            "lThighControl",
            "rArmControl",
            "lArmControl",
            "rShoulderControl",
            "lShoulderControl",
        };

        List<string> SourceNames = new List<string>()  {
            "rHandControl",
            "lHandControl",
            "rFootControl",
            "lFootControl",
            "eyeTargetControl",
            "rNippleControl",
            "lNippleControl",
            "rElbowControl",
            "lElbowControl",
            "rKneeControl",
            "lKneeControl",
            "rToeControl",
            "lToeControl",
            "rThighControl",
            "lThighControl",
            "rArmControl",
            "lArmControl",
            "rShoulderControl",
            "lShoulderControl",
        };

        List<string> KeyCodes = new List<string>() {
        "None",
        "space",
        "return",
        "up",
        "down",
        "left",
        "right",
        "left shift",
        "right shift",
        "left ctrl",
        "right ctrl",
        "left alt",
        "right alt",
        "tab",
        "backspace",
        "f2",
        "f3",
        "f4",
        "f5",
        "f6",
        "f7",
        "f8",
        "f9",
        "f10",
        "f11",
        "f12",
        "0",
        "1",
        "2",
        "3",
        "4",
        "5",
        "6",
        "7",
        "8",
        "9",
        "[0]",
        "[1]",
        "[2]",
        "[3]",
        "[4]",
        "[5]",
        "[6]",
        "[7]",
        "[8]",
        "[9]",
        "b",
        "c",
        "f",
        "g",
        "h",
        "i",
        "j",
        "k",
        "l",
        "o",
        "q",
        "r",
        "v",
        "y",
        "insert",
        "delete",
        "home",
        "end",
        "page up",
        "page down",
        "numlock",
        "caps lock",
        "scroll lock",
        "pause",
        };


        List<string> Mode = new List<string>() {
        "Hold",
        "Align",
        "Mirror",
        };

        List<string> AxisChoice = new List<string>() {
        "X",
        "Y",
        "Z",
        };

        private Dictionary<string, Dictionary<string, string>> ControlDics = new Dictionary<string, Dictionary<string, string>>();

        FreeControllerV3 SourceController;
        Transform CenterPointConnector;
        Transform SourceControllerConnector;
        Transform OtherSideConnector;
        Transform MirrorConnector;
        Rigidbody CenterPointRB;


        public override void Init()
        {
            try
            {
                #region Sliders


                CenterPoint = new JSONStorableStringChooser("Center Point", ControlNames, "control", "Center Point");
                RegisterStringChooser(CenterPoint);
                UIDynamicPopup CenterPointDyanamicPopup = CreateScrollablePopup(CenterPoint);
                CenterPointDyanamicPopup.popupPanelHeight = 1100f;
                CenterPoint.setCallbackFunction += (string Name) =>
                {
                    CenterPointAtomName = containingAtom.uid;
                };
                CenterPoint.setCallbackFunction(CenterPoint.val);

                UIDynamicButton SelectCenterFromScene;
                SelectCenterFromScene = CreateButton("Select CenterPoint From Scene", false);
                SelectCenterFromScene.button.onClick.AddListener(SelectFromSceneFunction);


                Source = new JSONStorableStringChooser("Source Control", SourceNames, "rHandControl", "Source Control");
                RegisterStringChooser(Source);
                UIDynamicPopup SourceDyanamicPopup = CreateScrollablePopup(Source, true);
                SourceDyanamicPopup.popupPanelHeight = 1100f;
                Source.setCallbackFunction += (string SourceName) =>
                {
                    SourceController = containingAtom.GetStorableByID(SourceName) as FreeControllerV3;
                };
                Source.setCallbackFunction(Source.val);

                UIDynamicButton MirroringBtn;
                MirroringBtn = CreateButton("Mirroring", true);
                MirroringBtn.button.onClick.AddListener(Mirroring);


                CreateSpacer(false).height = 20;


                Axis = new JSONStorableStringChooser("Align Axis", AxisChoice, "X", "Align Axis");
                RegisterStringChooser(Axis);
                CreateScrollablePopup(Axis);
                Axis.setCallbackFunction += (string AxisName) =>
                {
                    if (AxisName != "Custom")
                    {
                        PosX.val = AxisName == "X" ? "Mirror" : "Align";
                        PosY.val = AxisName == "Y" ? "Mirror" : "Align";
                        PosZ.val = AxisName == "Z" ? "Mirror" : "Align";
                        RotateX.val = AxisName == "X" ? "Align" : "Mirror";
                        RotateY.val = AxisName == "Y" ? "Align" : "Mirror";
                        RotateZ.val = AxisName == "Z" ? "Align" : "Mirror";
                    }
                    Axis.val = AxisName;


                };

                CreateSpacer(true).height = 60;

                MirrorParam = new JSONStorableBool("Mirroring Parameter", true);
                MirrorParam.storeType = JSONStorableParam.StoreType.Full;
                CreateToggle(MirrorParam, true);

                CreateSpacer(false).height = 20;
                CreateSpacer(true).height = 30;



                PosX = new JSONStorableStringChooser("Position X", Mode, "Mirror", "Position X");
                RegisterStringChooser(PosX);
                CreateScrollablePopup(PosX);
                PosX.setCallbackFunction += (string ModeName) =>
                {
                    Axis.val = "Custom";
                };


                PosY = new JSONStorableStringChooser("Position Y", Mode, "Align", "Position Y");
                RegisterStringChooser(PosY);
                CreateScrollablePopup(PosY);
                PosY.setCallbackFunction += (string ModeName) =>
                {
                    Axis.val = "Custom";
                };


                PosZ = new JSONStorableStringChooser("Position Z", Mode, "Align", "Position Z");
                RegisterStringChooser(PosZ);
                CreateScrollablePopup(PosZ);
                PosZ.setCallbackFunction += (string ModeName) =>
                {
                    Axis.val = "Custom";
                };
                RotateX = new JSONStorableStringChooser("Rotation X", Mode, "Align", "Rotation X");
                RegisterStringChooser(RotateX);
                CreateScrollablePopup(RotateX, true);

                RotateX.setCallbackFunction += (string ModeName) =>
                {
                    Axis.val = "Custom";
                };
                RotateY = new JSONStorableStringChooser("Rotation Y", Mode, "Mirror", "Rotation Y");
                RegisterStringChooser(RotateY);
                CreateScrollablePopup(RotateY, true);
                RotateY.setCallbackFunction += (string ModeName) =>
                {
                    Axis.val = "Custom";
                };
                RotateZ = new JSONStorableStringChooser("Rotation Z", Mode, "Mirror", "Rotation Z");
                RegisterStringChooser(RotateZ);
                CreateScrollablePopup(RotateZ, true);
                RotateZ.setCallbackFunction += (string ModeName) =>
                {
                    Axis.val = "Custom";
                };

                CreateSpacer(false).height = 20;


                MirroringKey = new JSONStorableStringChooser("Mirroring Key", KeyCodes, "None", "Mirroring Key");
                RegisterStringChooser(MirroringKey);
                UIDynamicPopup dp = CreateScrollablePopup(MirroringKey);
                dp.popupPanelHeight = 1100f;
                if (MirroringKey.val == "None")
                {
                    LoadShortCutKey();
                }


                UIDynamicButton savePresetButton = CreateButton("Save Key as Default");
                savePresetButton.button.onClick.AddListener(SaveShortCutKey);



                #endregion
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }


        }

        public void Update()
        {

            if (MirroringKey.val != "None")
            {
                if (Input.GetKeyDown(MirroringKey.val))
                {

                    FreeControllerV3 SourceControllerCheck = sc.GetSelectedController();
                    if (SourceControllerCheck)
                    {
                        Source.val = SourceControllerCheck.gameObject.name;
                    }
                    else
                    {
                        SuperController.LogMessage("Symmetrical Controller is not selected.");
                        return;
                    }
                    Mirroring();
                }
            }
        }

        private void SaveShortCutKey()
        {
            SimpleJSON.JSONClass Settings = new SimpleJSON.JSONClass();
            Settings.Add("Mirroring Key", new SimpleJSON.JSONData(MirroringKey.val));
            sc.SaveJSON(Settings, "Custom/Scripts/VaMJapan/PMsetting.json");
            SuperController.LogMessage("Mirroring Key was Saved");
        }

        private void LoadShortCutKey()
        {
            SimpleJSON.JSONClass LoadSettings = new SimpleJSON.JSONClass();

            try
            {
                LoadSettings = (SimpleJSON.JSONClass)sc.LoadJSON("Custom/Scripts/VaMJapan/PMsetting.json");
            }
            catch (Exception)
            {
                return;
            }

            if (LoadSettings != null)
            {
                MirroringKey.val = LoadSettings["Mirroring Key"];
            }
        }

        public void Mirroring()
        {

            string SourceControllerUID = SourceController.gameObject.name;
            string firstString = SourceControllerUID.Substring(0, 1);

            if (firstString == "r" || firstString == "l")
            {
                string otherSide = firstString == "r" ? "l" : "r";
                string otherSideControllerUID = otherSide + SourceControllerUID.TrimStart(firstString.ToCharArray());

                FreeControllerV3 OtherSideController = containingAtom.GetStorableByID(otherSideControllerUID) as FreeControllerV3;

                if (CenterPointConnector != null)
                {
                    UnityEngine.Object.DestroyImmediate(CenterPointConnector.gameObject);
                    CenterPointConnector = null;
                }

                GameObject CPgameObject = new GameObject();
                CenterPointConnector = CPgameObject.transform;


                string CenterPointRBName = CenterPointRB ? CenterPointRB.gameObject.name : "None";

                if (ControlNames.IndexOf(CenterPoint.val) == -1)
                {
                    CenterPointConnector.position = CenterPointRB.transform.position;
                    CenterPointConnector.rotation = CenterPointRB.transform.rotation;
                }
                else
                {
                    CenterPointRBName = CenterPointAtomName + ":" + CenterPoint.val;

                    CenterPointRB = sc.RigidbodyNameToRigidbody(CenterPointRBName);

                    CenterPointConnector.position = CenterPointRB.transform.position;
                    CenterPointConnector.rotation = CenterPointRB.transform.rotation;
                }




                Vector3 preCenterPointPos = CenterPointConnector.position;
                Quaternion preCenterPointQua = CenterPointConnector.rotation;


                if (SourceControllerConnector != null)
                {
                    UnityEngine.Object.DestroyImmediate(SourceControllerConnector.gameObject);
                    SourceControllerConnector = null;
                }
                GameObject SgameObject = new GameObject();
                SourceControllerConnector = SgameObject.transform;
                SourceControllerConnector.position = SourceController.transform.position;
                SourceControllerConnector.rotation = SourceController.transform.rotation;
                SourceControllerConnector.SetParent(CenterPointConnector);



                if (OtherSideConnector != null)
                {
                    UnityEngine.Object.DestroyImmediate(OtherSideConnector.gameObject);
                    OtherSideConnector = null;
                }
                GameObject OSgameObject = new GameObject();
                OtherSideConnector = OSgameObject.transform;
                OtherSideConnector.position = OtherSideController.transform.position;
                OtherSideConnector.rotation = OtherSideController.transform.rotation;
                Vector3 OSRotationVector = OtherSideConnector.eulerAngles;
                OtherSideConnector.SetParent(CenterPointConnector);

                CenterPointConnector.position = new Vector3(0f, 0f, 0f);
                CenterPointConnector.rotation = Quaternion.Euler(new Vector3(0f, 0f, 0f));


                if (MirrorConnector != null)
                {
                    UnityEngine.Object.DestroyImmediate(MirrorConnector.gameObject);
                    MirrorConnector = null;
                }
                GameObject MgameObject = new GameObject();
                MirrorConnector = MgameObject.transform;
                MirrorConnector.position = CenterPointConnector.position + (CenterPointConnector.position - SourceControllerConnector.position);
                Vector3 SCRrotationVectror = SourceControllerConnector.eulerAngles;
                Vector3 MirrorVectror = -SCRrotationVectror;
                MirrorConnector.SetParent(CenterPointConnector);

                Vector3 tempVector = MirrorConnector.position;

                if (PosX.val == "Hold")
                {
                    tempVector.x = OtherSideConnector.position.x;
                }
                else if (PosX.val == "Align")
                {

                    tempVector.x = SourceControllerConnector.position.x;
                }

                if (PosY.val == "Hold")
                {
                    tempVector.y = OtherSideConnector.position.y;
                }
                else if (PosY.val == "Align")
                {

                    tempVector.y = SourceControllerConnector.position.y;
                }
                if (PosZ.val == "Hold")
                {
                    tempVector.z = OtherSideConnector.position.z;
                }
                else if (PosZ.val == "Align")
                {

                    tempVector.z = SourceControllerConnector.position.z;
                }




                if (RotateX.val == "Hold")
                {
                    MirrorVectror.x = OSRotationVector.x;
                }
                else if (RotateX.val == "Align")
                {

                    MirrorVectror.x = SCRrotationVectror.x;
                }


                if (RotateY.val == "Hold")
                {
                    MirrorVectror.y = OSRotationVector.y;
                }
                else if (RotateY.val == "Align")
                {

                    MirrorVectror.y = SCRrotationVectror.y;
                }
                if (RotateZ.val == "Hold")
                {
                    MirrorVectror.z = OSRotationVector.z;
                }
                else if (RotateZ.val == "Align")
                {

                    MirrorVectror.z = SCRrotationVectror.z;
                }

                MirrorConnector.position = tempVector;
                MirrorConnector.rotation = Quaternion.Euler(MirrorVectror);

                CenterPointConnector.position += preCenterPointPos;
                CenterPointConnector.rotation = preCenterPointQua;



                OtherSideController.transform.position = MirrorConnector.position;
                OtherSideController.transform.rotation = MirrorConnector.rotation;

                if (MirrorParam.val)
                {
                    OtherSideController.jointRotationDriveDamper = SourceController.jointRotationDriveDamper;
                    OtherSideController.jointRotationDriveMaxForce = SourceController.jointRotationDriveMaxForce;
                    OtherSideController.jointRotationDriveSpring = SourceController.jointRotationDriveSpring;
                    OtherSideController.jointRotationDriveXTarget = -SourceController.jointRotationDriveXTarget;
                    OtherSideController.jointRotationDriveXTargetAdditional = -SourceController.jointRotationDriveXTargetAdditional;
                    OtherSideController.jointRotationDriveYTarget = -SourceController.jointRotationDriveYTarget;
                    OtherSideController.jointRotationDriveYTargetAdditional = -SourceController.jointRotationDriveYTargetAdditional;
                    OtherSideController.jointRotationDriveZTarget = SourceController.jointRotationDriveZTarget;
                    OtherSideController.jointRotationDriveZTargetAdditional = SourceController.jointRotationDriveZTargetAdditional;

                    if (SourceControllerUID.TrimStart(firstString.ToCharArray()) == "KneeControl" || SourceControllerUID.TrimStart(firstString.ToCharArray()) == "FootControl" || SourceControllerUID.TrimStart(firstString.ToCharArray()) == "ToeControl" || SourceControllerUID.TrimStart(firstString.ToCharArray()) == "ThighControl")
                    {
                        OtherSideController.jointRotationDriveXTarget = SourceController.jointRotationDriveXTarget;
                        OtherSideController.jointRotationDriveXTargetAdditional = SourceController.jointRotationDriveXTargetAdditional;
                        OtherSideController.jointRotationDriveZTarget = -SourceController.jointRotationDriveZTarget;
                        OtherSideController.jointRotationDriveZTargetAdditional = -SourceController.jointRotationDriveZTargetAdditional;
                    }

                    if (SourceControllerUID.TrimStart(firstString.ToCharArray()) == "ElbowControl")
                    {
                        OtherSideController.jointRotationDriveYTarget = SourceController.jointRotationDriveYTarget;
                        OtherSideController.jointRotationDriveYTargetAdditional = SourceController.jointRotationDriveYTargetAdditional;
                        OtherSideController.jointRotationDriveZTarget = -SourceController.jointRotationDriveZTarget;
                        OtherSideController.jointRotationDriveZTargetAdditional = -SourceController.jointRotationDriveZTargetAdditional;

                    }

                    OtherSideController.RBComplyJointRotationDriveSpring = SourceController.RBComplyJointRotationDriveSpring;
                    OtherSideController.RBComplyPositionDamper = SourceController.RBComplyPositionDamper;
                    OtherSideController.RBComplyPositionSpring = SourceController.RBComplyPositionSpring;
                    OtherSideController.RBComplyRotationDamper = SourceController.RBComplyRotationDamper;
                    OtherSideController.RBComplyRotationSpring = SourceController.RBComplyRotationSpring;
                    OtherSideController.RBHoldPositionDamper = SourceController.RBHoldPositionDamper;
                    OtherSideController.RBHoldPositionMaxForce = SourceController.RBHoldPositionMaxForce;
                    OtherSideController.RBHoldPositionSpring = SourceController.RBHoldPositionSpring;
                    OtherSideController.RBHoldRotationDamper = SourceController.RBHoldRotationDamper;
                    OtherSideController.RBHoldRotationMaxForce = SourceController.RBHoldRotationMaxForce;
                    OtherSideController.RBHoldRotationSpring = SourceController.RBHoldRotationSpring;
                    OtherSideController.RBLinkPositionDamper = SourceController.RBLinkPositionDamper;
                    OtherSideController.RBLinkPositionMaxForce = SourceController.RBLinkPositionMaxForce;
                    OtherSideController.RBLinkPositionSpring = SourceController.RBLinkPositionSpring;
                    OtherSideController.RBLinkRotationDamper = SourceController.RBLinkRotationDamper;
                    OtherSideController.RBLinkRotationMaxForce = SourceController.RBLinkRotationMaxForce;
                    OtherSideController.RBLinkRotationSpring = SourceController.RBLinkRotationSpring;
                    OtherSideController.RBLockPositionDamper = SourceController.RBLockPositionDamper;
                    OtherSideController.RBLockPositionMaxForce = SourceController.RBLockPositionMaxForce;
                    OtherSideController.RBLockPositionSpring = SourceController.RBLockPositionSpring;
                    OtherSideController.RBLockRotationDamper = SourceController.RBLockRotationDamper;
                    OtherSideController.RBLockRotationMaxForce = SourceController.RBLockRotationMaxForce;
                    OtherSideController.RBLockRotationSpring = SourceController.RBLockRotationSpring;
                    OtherSideController.linkToRB = SourceController.linkToRB;
                    OtherSideController.currentPositionState = SourceController.currentPositionState;
                    OtherSideController.currentRotationState = SourceController.currentRotationState;

                }
            }
            else
            {
                SuperController.LogMessage("Symmetrical Controller is not selected.");
            }
        }

        protected void SelectFromSceneFunction()
        {
            sc.SelectModeRigidbody(new SuperController.SelectRigidbodyCallback(this.SetController));
        }

        string CenterPointAtomName = "None";
        protected void SetController(Rigidbody rb)
        {
            Atom CenterPointAtom = null;
            FreeControllerV3 component = rb.GetComponent<FreeControllerV3>();
            if (component != null)
            {
                CenterPointAtom = component.containingAtom;
            }
            else
            {
                ForceReceiver component2 = rb.GetComponent<ForceReceiver>();
                if (component2 != null)
                {
                    CenterPointAtom = component2.containingAtom;
                }
            }
            if (CenterPointAtom != null)
            {
                CenterPointAtomName = CenterPointAtom.uid;
            }

            CenterPointRB = rb;

            string tempStr = CenterPointAtomName + ":" + rb.name;
            CenterPoint.val = tempStr;

        }
    }
}